package com.philvarner.clamavj.test;

import static com.philvarner.clamavj.ScanResult.RESPONSE_OK;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import org.junit.Before;
import org.junit.Test;

import com.philvarner.clamavj.ClamScan;
import com.philvarner.clamavj.ScanResult;
import com.philvarner.clamavj.ScanResult.Status;

public class ClamScanTestCase {

    private ClamScan scanner;

    @Before
    public void setUp() {
        //143.106.20.35
    	scanner = new ClamScan("127.0.0.1", 3310, 120000);
    }

    @Test
    public void testSuccess() throws Exception {

        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < 10000; i++) {
            sb.append("abcde12345");
        }

        InputStream is = new ByteArrayInputStream(sb.toString().getBytes());
        assertNotNull(is);
        ScanResult result = scanner.scan(is);

        printScanStatus(result, "testSuccess");

        assertEquals(Status.PASSED, result.getStatus());
        assertEquals(RESPONSE_OK, result.getResult());
    }

    private void printScanStatus(ScanResult result, String metodo) {
        StringBuilder stb = new StringBuilder();
        stb.append("\n========[RESULT]========\n");
        stb.append("METODO: " + metodo);
        stb.append("\n result string...: " + result.getResult());
        stb.append("\n result signature: " + result.getSignature());
        stb.append("\n result status...: " + result.getStatus());
        stb.append("\n result exception: " + result.getException());
        stb.append("\n========================\n\n");
        System.out.println(stb.toString());
    }

    @Test
    public void testVirus() throws Exception {
        InputStream is = new ByteArrayInputStream(
                "X5O!P%@AP[4\\PZX54(P^)7CC)7}$EICAR-STANDARD-ANTIVIRUS-TEST-FILE!$H+H*".getBytes());
        assertNotNull(is);
        ScanResult result = scanner.scan(is);

        printScanStatus(result, "testVirus");

        assertEquals(Status.FAILED, result.getStatus());
        assertEquals("stream: Eicar-Test-Signature FOUND", result.getResult());
        assertEquals("Eicar-Test-Signature", result.getSignature());
    }

    @Test
    public void testVirusAsByteArray() throws Exception {
        byte[] bytes = "X5O!P%@AP[4\\PZX54(P^)7CC)7}$EICAR-STANDARD-ANTIVIRUS-TEST-FILE!$H+H*".getBytes();
        ScanResult result = scanner.scan(bytes);

        printScanStatus(result, "testVirusAsByteArray");        

        assertEquals(Status.FAILED, result.getStatus());
        assertEquals("stream: Eicar-Test-Signature FOUND", result.getResult());
        assertEquals("Eicar-Test-Signature", result.getSignature());
    }

    @Test
    public void test_virus_from_file() throws Exception {

        int byteCount;
        byte bytes[];
        given: {
            //File f = new File("src/test/resources/eicar.txt");
        	File f = new File("/developer/docs/pessoal/Clayton_PropostaAssinada.pdf");
            FileInputStream fis = new FileInputStream(f);
            bytes = new byte[(int) f.length()];
            byteCount = fis.read(bytes);
        }

        ScanResult result;
        when: {
            result = scanner.scan(bytes);

            printScanStatus(result, "test_virus_from_file");
        }

        then: {
            assertTrue(byteCount > 0);
            assertEquals(Status.FAILED, result.getStatus());
            assertEquals("stream: Eicar-Test-Signature FOUND", result.getResult());
            assertEquals("Eicar-Test-Signature", result.getSignature());
        }
    }

    @Test
    public void testNoArgConstructor() throws Exception {
        scanner = new ClamScan();
        scanner.setHost("localhost");
        scanner.setPort(3310);
        scanner.setTimeout(60000);

        byte[] bytes = "X5O!P%@AP[4\\PZX54(P^)7CC)7}$EICAR-STANDARD-ANTIVIRUS-TEST-FILE!$H+H*".getBytes();
        ScanResult result = scanner.scan(bytes);

        printScanStatus(result, "testNoArgConstructor");        

        assertEquals(Status.FAILED, result.getStatus());
        assertEquals("stream: Eicar-Test-Signature FOUND", result.getResult());
        assertEquals("Eicar-Test-Signature", result.getSignature());
    }

    @Test
    public void testMultipleOfChunkSize() throws Exception {
        InputStream is = new ArbitraryInputStream(ClamScan.CHUNK_SIZE);
        assertNotNull(is);
        ScanResult result = scanner.scan(is);

        printScanStatus(result, "testMultipleOfChunkSize");        

        assertEquals(Status.PASSED, result.getStatus());
        assertEquals(RESPONSE_OK, result.getResult());
    }

    @Test
    public void testTooLarge() throws Exception {
        InputStream is = new ArbitraryInputStream();
        assertNotNull(is);
        ScanResult result = scanner.scan(is);

        printScanStatus(result, "testTooLarge");        

        assertEquals(result.getResult(), Status.ERROR, result.getStatus());
        assertEquals(ScanResult.RESPONSE_SIZE_EXCEEDED, result.getResult());
    }

    @Test
    public void testStats() throws Exception {
        String result = scanner.stats();

        System.out.println("testStats: \n" + result);
        
        assertTrue("didn't contain POOLS: \n" + result, result.contains("POOLS:") || result.contains("STATE:"));
    }

    @Test
    public void testPing() throws Exception {
    	System.out.println("testPing");
        assertTrue(scanner.ping());
    }
}